#include <iostream>
#include <occi.h>

#include <cstring>

int main() {
    std::string userName = "HR";
    std::string password = "oracle";
    std::string connectionString = "//192.168.1.150:1521/orcl";

    oracle::occi::Environment* env = oracle::occi::Environment::createEnvironment(oracle::occi::Environment::THREADED_UNMUTEXED);
    oracle::occi::Connection* conn;

    try {
        conn = env->createConnection(userName, password, connectionString);
    } catch(oracle::occi::SQLException& ex) {
        oracle::occi::Environment::terminateEnvironment(env);
        std::cout << "CONNECTION ERROR: " << ex.getMessage() << std::endl;
        return 1;
    }

    /*
    std::string version = conn->getServerVersion();
    std::cout << "VERSION: " << version << std::endl;
    */

    std::string query = "SELECT \"TEXT\", \"NAME\", \"STEXT\" FROM \"TEST1\"";
    oracle::occi::Statement* stmt = conn->createStatement(query);
    oracle::occi::ResultSet* rs = NULL;

    try {
        oracle::occi::Statement::Status status = stmt->execute();
        if (status == oracle::occi::Statement::RESULT_SET_AVAILABLE) {
            rs = stmt->getResultSet();
        } else {
            std::cout << "OTHER RESPONSE" << std::endl;
        }
    } catch(oracle::occi::SQLException& ex) {
        std::cout << "SQL ERROR: " << ex.getMessage() << std::endl;
    }

    if (rs != NULL) {
        std::cout << "GOT RESULTSET" << std::endl;

        std::vector<oracle::occi::MetaData> columns = rs->getColumnListMetaData();
        std::cout << columns.size() << " COLUMNS" << std::endl;
        for (unsigned int i=0, limiti=columns.size(); i < limiti; i++) {
            int type = columns[i].getInt(oracle::occi::MetaData::ATTR_DATA_TYPE);

            std::cout << "\t" << columns[i].getString(oracle::occi::MetaData::ATTR_NAME) << ": ";
            if (type == oracle::occi::OCCI_TYPECODE_BLOB) {
                std::cout << "BLOB";
            } else if (type == oracle::occi::OCCI_TYPECODE_VARCHAR2) {
                std::cout << "VARCHAR2";
            } else if (type == oracle::occi::OCCI_TYPECODE_VARCHAR) {
                std::cout << "VARCHAR";
            } else {
                std::cout << "unknown";
            }

            std::cout << " - " << columns[i].getInt(oracle::occi::MetaData::ATTR_DATA_SIZE);
            std::cout << std::endl;

        }

        // INSERT INTO "TEST1"("NAME", "STEXT", "TEXT") VALUES('test1.1', CONCAT(CONCAT('Hello', CHR(0)), 'World'), rawtohex(CONCAT(CONCAT('Hello', CHR(0)), 'World')));
        try {
            while (rs->next()) {
                std::cout << "DATA SIZE: " << rs->getMaxColumnSize(2) << std::endl;
                std::string v = rs->getString(3);
                const char* s2 = v.c_str();
                unsigned long l = v.length();
                unsigned char* s = new unsigned char[l];
                memcpy(s, s2, l);
                unsigned long i=0;
                std::cout << "S(" << l << "): ";
                for (i=0; i < l; i++) {
                    std::cout << " - " << s[i];
                }
                std::cout << std::endl;
                delete [] s;

                std::cout << rs->getString(2) << " - " << rs->getString(3) << std::endl;

                std::cout << " | ";
                oracle::occi::Blob blob = rs->getBlob(1);
                unsigned int length = blob.length();
                s = new unsigned char[length];
                blob.read(length, s, length);
                std::cout << "S(" << length << "): ";
                for (i=0; i < length; i++) {
                    std::cout << " - " << s[i];
                }
                std::cout << std::endl;
                delete [] s;
            }
        } catch(oracle::occi::SQLException& ex) {
            std::cout << "ERROR: " << ex.getMessage() << std::endl;
        }

        stmt->closeResultSet(rs);
    }

    conn->terminateStatement(stmt);

    env->terminateConnection(conn);
    oracle::occi::Environment::terminateEnvironment(env);

    return 0;
}
